@@ -15,14 +15,19 @@ struct __locale_map {
 	const struct __locale_map *next;
 };
 
-extern const struct __locale_map __c_dot_utf8;
-extern const struct __locale_struct __c_locale;
-extern const struct __locale_struct __c_dot_utf8_locale;
+extern hidden const struct __locale_map __c_dot_utf8;
+extern hidden const struct __locale_struct __c_locale;
+extern hidden const struct __locale_struct __c_dot_utf8_locale;
 
-const struct __locale_map *__get_locale(int, const char *);
-const char *__mo_lookup(const void *, size_t, const char *);
-const char *__lctrans(const char *, const struct __locale_map *);
-const char *__lctrans_cur(const char *);
+hidden const struct __locale_map *__get_locale(int, const char *);
+hidden const char *__mo_lookup(const void *, size_t, const char *);
+hidden const char *__lctrans(const char *, const struct __locale_map *);
+hidden const char *__lctrans_cur(const char *);
+hidden const char *__lctrans_impl(const char *, const struct __locale_map *);
+hidden int __loc_is_allocated(locale_t);
+hidden char *__gettextdomain(void);
+
+#define LOC_MAP_FAILED ((const struct __locale_map *)-1)
 
 #define LCTRANS(msg, lc, loc) __lctrans(msg, (loc)->cat[(lc)])
 #define LCTRANS_CUR(msg) __lctrans_cur(msg)
